Name:           c4core
Summary:        C++ core utilities
Version:        0.1.11
%global so_version 0.1.11
Release:    4%{?dist}

URL:            https://github.com/biojppm/c4core
License:        %{shrink:
                MIT AND
                BSL-1.0 AND
                BSD-2-Clause AND
                (Apache-2.0 OR MIT OR BSL-1.0)
                }
Source0:         %{url}/archive/v%{version}/c4core-%{version}.tar.gz

BuildRequires:  gcc-c++
BuildRequires:  cmake
BuildRequires:  c4project
BuildRequires:  ninja-build

BuildRequires:  debugbreak-devel
BuildRequires:  debugbreak-static
BuildRequires:  fast_float-devel
BuildRequires:  fast_float-static
BuildRequires:  doctest-devel
BuildRequires:  doctest-static

%global common_description %{expand:
c4core is a library of low-level C++ utilities, written with low-latency
projects in mind.

Some of the utilities provided by c4core have already equivalent functionality
in the C++ standard, but they are provided as the existing C++ equivalent may
be insufficient (eg, std::string_view), inefficient (eg, std::string), heavy
(eg streams), or plainly unusable on some platforms/projects, (eg exceptions);
some other utilities have equivalent under consideration for C++
standardization; and yet some other utilities have (to my knowledge) no
equivalent under consideration.}

%description %{common_description}


%package devel
Summary:        %{summary}

Requires:       %{name} = %{version}-%{release}

Requires:       debugbreak-devel
Requires:       fast_float-devel

Provides:       bundled(ag-random) = 0^20180702git6dc334c
Provides:       bundled(SG14) = 0^20190524git3a3b806

%description devel %{common_description}


%prep
%autosetup -n c4core-%{version}


rm -rvf cmake
ln -s '%{_datadir}/cmake/c4project' cmake

sed -r -i \
    -e 's/(LIBS.*)\bdoctest\b/\1/' \
    -e 's/(c4_setup_testing\()DOCTEST\)/\1\)/' \
    test/CMakeLists.txt

if [ "$(stat -c '%s' changelog/current.md)" = '0' ]
then
  rm -vf changelog/current.md
fi

rm -rvf src/c4/ext/debugbreak
mkdir src/c4/ext/debugbreak
ln -sv %{_includedir}/debugbreak.h src/c4/ext/debugbreak/

# fast_float
#
# The build system expects to produce an amalgamated header, fast_float_all.h,
# from the bundled fast_float. We therefore use a symbolic link to preserve the
# original bundled header path. The amalgamated header will be produced from
# the *system* fast_float headers. Once the amalgamated header is produced, we
# again replace it with a symbolic link to the main system fast_float header,
# thereby fully unbundling fast_float.
rm -rv src/c4/ext/fast_float
mkdir -p src/c4/ext/fast_float/include
ln -sv %{_includedir}/fast_float src/c4/ext/fast_float/include/
# Replace amalgamated single-file header produced by build system with one that
# trivially includes the main system fast_float header.
cat > src/c4/ext/fast_float_all.h <<EOF
#include <fast_float/fast_float.h>
EOF


%build
%cmake -GNinja \
  -DC4CORE_CACHE_DOWNLOAD_DOCTEST:PATH=/ \
  -DC4CORE_BUILD_TESTS=ON
%cmake_build


%install
%cmake_install
if [ '%{_libdir}' != '%{_prefix}/lib' ]
then
  mkdir -p '%{buildroot}%{_libdir}'
  mv -v %{buildroot}%{_prefix}/lib/libc4core.so* '%{buildroot}%{_libdir}/'
  mkdir -p '%{buildroot}%{_libdir}/cmake'
  mv -v %{buildroot}%{_prefix}/lib/cmake/c4core '%{buildroot}%{_libdir}/cmake/'
  find %{buildroot}%{_libdir}/cmake/c4core -type f -name '*.cmake' -print0 |
    xargs -r -t -0 sed -r -i "s@/lib/@/$(basename '%{_libdir}')/@"
fi


ln -svf '%{_includedir}/debugbreak.h' \
    '%{buildroot}%{_includedir}/c4/ext/debugbreak/'


%check
%cmake_build --target c4core-test-run-verbose


%files
%license LICENSE.txt LICENSE-BOOST.txt
%doc README.md
%doc ROADMAP.md
%doc changelog/
%{_libdir}/libc4core.so.%{so_version}


%files devel
%{_includedir}/c4/
%{_libdir}/libc4core.so
%{_libdir}/cmake/c4core/


%changelog
* Fri Aug 09 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.1.11-4
- bump version

* Fri Aug 09 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.1.11-3
- fix build error from c4fs

* Fri Aug 09 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.1.11-2
- fix build error

* Fri Apr 12 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.1.11-1
- initial build
